@@ -529,20 +529,13 @@ class Install(install_misc.InstallBase):
                 self._db.subst(template, substr, data)
 
         for plugin in self.plugins:
-            if plugin.NAME == 'migrationassistant' and \
-                'UBIQUITY_MIGRATION_ASSISTANT' not in os.environ:
-                    continue
             self.next_region()
             # set a generic info message in case plugin doesn't provide one
             self.db.progress('INFO', 'ubiquity/install/title')
             inst = plugin.Install(None, db=self.db)
             ret = inst.install(self.target, Progress(self.db))
             if ret:
-                if plugin.NAME == 'migrationassistant':
-                    self.db.input('critical', 'ubiquity/install/broken_migration')
-                    self.db.go()
-                else:
-                    raise install_misc.InstallStepError("Plugin %s failed with code %s" % (plugin.NAME, ret))
+                raise install_misc.InstallStepError("Plugin %s failed with code %s" % (plugin.NAME, ret))
 
     def configure_apt(self):
         """Configure /etc/apt/sources.list."""
@@ -778,6 +771,7 @@ class Install(install_misc.InstallBase):
         self.progress_region(1, 2)
         if install_kernels:
             self.do_install(install_kernels)
+            install_misc.record_installed(install_kernels)
             if new_kernel_pkg:
                 cache = Cache()
                 cached_pkg = install_misc.get_cache_pkg(cache, new_kernel_pkg)
@@ -1453,6 +1447,7 @@ class Install(install_misc.InstallBase):
 
         if arch in ('amd64', 'i386'):
             for pkg in ('grub', 'grub-pc', 'grub-efi', 'grub-efi-amd64',
+                        'grub-efi-amd64-signed', 'shim-signed',
                         'lilo'):
                 if pkg not in keep:
                     difference.add(pkg)
