@@ -34,7 +34,6 @@ import dbus
 
 # kde gui specifics
 from PyQt4 import QtCore, QtGui, uic
-from PyKDE4 import kdeui, kdecore
 
 #import all our custom kde components
 from ubiquity.frontend.kde_components import ProgressDialog, SqueezeLabel
@@ -56,8 +55,8 @@ LOCALEDIR = "/usr/share/locale"
 #currently using for testing, will remove
 UIDIR = os.path.join(PATH, 'qt')
 
-class UbiquityUI(kdeui.KMainWindow):
 
+class UbiquityUI(QtGui.QMainWindow):
     def __init__(self, parent=None):
         QtGui.QMainWindow.__init__(self, parent)
         uic.loadUi(os.path.join(UIDIR, "app.ui"), self)
@@ -123,7 +122,7 @@ class Controller(ubiquity.frontend.base.Controller):
 
     def allow_go_backward(self, allowed):
         self._wizard.allow_go_backward(allowed)
-    
+
     def allow_change_step(self, allowed):
         self._wizard.allow_change_step(allowed)
 
@@ -165,35 +164,12 @@ class Wizard(BaseFrontend):
         self.previous_excepthook = sys.excepthook
         sys.excepthook = self.excepthook
 
-        appName     = "kubuntu-ubiquity"
-        catalog     = ""
-        programName = kdecore.ki18n ("Installer")
-        ver         = "1.0"
-        description = kdecore.ki18n ("Live CD Installer for Kubuntu")
-        rights      = kdecore.KAboutData.License_GPL
-        copy        = kdecore.ki18n ("(c) 2006 Canonical Ltd")
-        text        = kdecore.ki18n ("none")
-        homePage    = "http://wiki.kubuntu.org/KubuntuUbiquity"
-        bugEmail    = "jriddell@ubuntu.com"
-
-        about = kdecore.KAboutData (appName, catalog, programName, ver, description,
-                            rights, copy, text, homePage, bugEmail)
-        about.addAuthor(kdecore.ki18n("Jonathan Riddell"), kdecore.KLocalizedString() ,"jriddell@ubuntu.com")
-        about.addAuthor(kdecore.ki18n("Roman Shtylman"), kdecore.KLocalizedString() ,"shtylman@gmail.com")
-        kdecore.KCmdLineArgs.init([""],about)
-
-        # KApplication won't initialise if real UID != effective UID.  On
-        # the other hand, we can't talk to D-Bus unless the effective user
-        # is the live CD user.  Oh dear.  The solution is to use saved IDs:
-        # if we hide our rootliness in the saved IDs, then neither
-        # KApplication nor D-Bus will spot it.
-        misc.drop_privileges_save()
-        try:
-            self.app = kdeui.KApplication()
-            self.app.setStyleSheet(file(os.path.join(UIDIR, "style.qss")).read())
-        finally:
-            misc.regain_privileges_save()
+        self.app = QtGui.QApplication([])
+        with open(os.path.join(UIDIR, "style.qss")) as style:
+            self.app.setStyleSheet(style.read())
 
+        self.app.setWindowIcon(QtGui.QIcon(
+         "/usr/share/icons/hicolor/128x128/apps/ubiquity.png"))
         import dbus.mainloop.qt
         dbus.mainloop.qt.DBusQtMainLoop(set_as_default=True)
 
@@ -311,7 +287,8 @@ class Wizard(BaseFrontend):
         self.socketNotifierException = {}
         self.debconf_callbacks = {}    # array to keep callback functions needed by debconf file descriptors
 
-        self.ui.setWindowIcon(kdeui.KIcon("ubiquity"))
+        self.ui.setWindowIcon(QtGui.QIcon(
+         "/usr/share/icons/hicolor/128x128/apps/ubiquity.png"))
         self.allow_go_backward(False)
 
         if not 'UBIQUITY_AUTOMATIC' in os.environ:
@@ -326,7 +303,8 @@ class Wizard(BaseFrontend):
             self.ui.setWindowTitle(self.get_string('oem_config_title'))
         elif self.oem_user_config:
             self.ui.setWindowTitle(self.get_string('oem_user_config_title'))
-            self.ui.setWindowIcon(kdeui.KIcon("preferences-system"))
+            self.ui.setWindowIcon(QtGui.QIcon(
+          "/usr/share/icons/oxygen/128x128/categories/preferences-system.png"))
             flags = self.ui.windowFlags() ^ QtCore.Qt.WindowMinMaxButtonsHint
             if hasattr(QtCore.Qt, 'WindowCloseButtonHint'):
                 flags = flags ^ QtCore.Qt.WindowCloseButtonHint
@@ -336,21 +314,27 @@ class Wizard(BaseFrontend):
             self.ui.install_process_label.hide()
             self.ui.breadcrumb_install.hide()
 
-        self.forwardIcon = kdeui.KIcon("go-next")
+        self.forwardIcon = QtGui.QIcon(
+         "/usr/share/icons/oxygen/128x128/actions/go-next.png")
         self.ui.next.setIcon(self.forwardIcon)
 
         #Used for the last step
-        self.applyIcon = kdeui.KIcon("dialog-ok-apply")
+        self.applyIcon = QtGui.QIcon(
+         "/usr/share/icons/oxygen/128x128/actions/dialog-ok-apply.png")
 
-        backIcon = kdeui.KIcon("go-previous")
+        backIcon = QtGui.QIcon(
+         "/usr/share/icons/oxygen/128x128/actions/go-previous.png")
         self.ui.back.setIcon(backIcon)
 
-        quitIcon = kdeui.KIcon("dialog-close")
+        quitIcon = QtGui.QIcon(
+         "/usr/share/icons/oxygen/48x48/actions/dialog-close.png")
         self.ui.quit.setIcon(quitIcon)
 
         self.ui.progressBar.hide()
         self.ui.progressCancel.hide()
 
+        misc.add_connection_watch(self.network_change)
+
     def excepthook(self, exctype, excvalue, exctb):
         """Crash handler."""
 
@@ -374,11 +358,31 @@ class Wizard(BaseFrontend):
         else:
             dialog = QtGui.QDialog(self.ui)
             uic.loadUi("%s/crashdialog.ui" % UIDIR, dialog)
-            dialog.beastie_url.setOpenExternalLinks(True)
             dialog.crash_detail.setText(tbtext)
             dialog.exec_()
             sys.exit(1)
 
+    def network_change(self, online=False):
+        from PyQt4.QtCore import QTimer, SIGNAL
+        if not online:
+            self.set_online_state(False)
+            return
+        QTimer.singleShot(300, self.check_returncode)
+        self.timer = QTimer(self.ui)
+        self.timer.connect(self.timer, SIGNAL("timeout()"), self.check_returncode)
+        self.timer.start(300)
+
+    def check_returncode(self, *args):
+        from PyQt4.QtCore import SIGNAL
+        if not BaseFrontend.check_returncode(self, args):
+            self.timer.disconnect(self.timer, SIGNAL("timeout()"),
+                self.check_returncode)
+
+    def set_online_state(self, state):
+        for p in self.pages:
+            if hasattr(p.ui, 'plugin_set_online_state'):
+                p.ui.plugin_set_online_state(state)
+
     # Disable the KDE media notifier to avoid problems during partitioning.
     def disable_volume_manager(self):
         # FIXME, medianotifier unload port to KDE 4"
@@ -519,7 +523,7 @@ class Wizard(BaseFrontend):
         slideshow_locale = self.slideshow_get_available_locale(slideshow_dir,
                                                                self.locale)
         slideshow_main = slideshow_dir + '/slides/index.html'
-        if not os.path.exists(slideshow_main):
+        if not os.path.exists(slideshow_main) or self.hide_slideshow:
             self.ui.pageMode.hide()
             return
 
@@ -625,6 +629,8 @@ class Wizard(BaseFrontend):
             core_names.append('ubiquity/text/breadcrumb_install')
             core_names.append('ubiquity/text/release_notes_only')
             core_names.append('ubiquity/text/update_installer_only')
+            core_names.append('ubiquity/text/USB')
+            core_names.append('ubiquity/text/CD')
             for stock_item in ('cancel', 'close', 'go-back', 'go-forward',
                                'ok', 'quit', 'yes', 'no'):
                 core_names.append('ubiquity/imported/%s' % stock_item)
@@ -836,42 +842,6 @@ class Wizard(BaseFrontend):
             step_index = 0
         return str(self.stackLayout.widget(step_index).objectName())
 
-    def add_history(self, page, widget):
-        history_entry = (page, widget)
-        if self.history:
-            # We may have skipped past child pages of the component.  Remove
-            # the history between the page we're on and the end of the list in
-            # that case.
-            if history_entry in self.history:
-                idx = self.history.index(history_entry)
-                if idx + 1 < len(self.history):
-                    self.history = self.history[:idx+1]
-                    return # The page is now effectively a dup
-            # We may have either jumped backward or forward over pages.
-            # Correct history in that case
-            new_index = self.pages.index(page)
-            old_index = self.pages.index(self.history[-1][0])
-            # First, pop if needed
-            if new_index < old_index:
-                while self.history[-1][0] != page and len(self.history) > 1:
-                    self.pop_history()
-            # Now push fake history if needed
-            i = old_index + 1
-            while i < new_index:
-                for _ in self.pages[i].widgets:
-                    self.history.append((self.pages[i], None))
-                i += 1
-
-            if history_entry == self.history[-1]:
-                return # Don't add the page if it's a dup
-        self.history.append(history_entry)
-
-    def pop_history(self):
-        if len(self.history) < 2:
-            return self.pagesindex
-        self.history.pop()
-        return self.pages.index(self.history[-1][0])
-
     def set_current_page(self, current):
         widget = self.stackLayout.widget(current)
         if self.stackLayout.currentWidget() == widget:
@@ -944,14 +914,18 @@ class Wizard(BaseFrontend):
         abortTitle = self.get_string("warning_dialog")
         yes = self.get_string('yes', prefix='ubiquity/imported')
         no = self.get_string('no', prefix='ubiquity/imported')
-        if yes and no:
-            yes = kdeui.KGuiItem(yes.replace('_', '&', 1))
-            no = kdeui.KGuiItem(no.replace('_', '&', 1))
-            args = (self.ui, abortTitle, warning_dialog_label, yes, no)
-        else:
-            args = (self.ui, abortTitle, warning_dialog_label)
-        response = kdeui.KMessageBox.questionYesNo(*args)
-        if response == kdeui.KMessageBox.Yes:
+        messageBox = QtGui.QMessageBox()
+        messageBox.setWindowTitle(abortTitle)
+        messageBox.setText(warning_dialog_label)
+        messageBox.setStandardButtons(QtGui.QMessageBox.Yes |
+                                                          QtGui.QMessageBox.No)
+        messageBox.setDefaultButton(QtGui.QMessageBox.Yes)
+        messageBox.button(QtGui.QMessageBox.Yes).setText(
+                                                      yes.replace('_', '&', 1))
+        messageBox.button(QtGui.QMessageBox.No).setText(
+                                                       no.replace('_', '&', 1))
+        response = messageBox.exec_()
+        if response == QtGui.QMessageBox.Yes:
             self.current_page = None
             self.quit()
             return True
@@ -1159,7 +1133,6 @@ class Wizard(BaseFrontend):
             # dialog instead.
             dialog = QtGui.QDialog(self.ui)
             uic.loadUi("%s/crashdialog.ui" % UIDIR, dialog)
-            dialog.beastie_url.setOpenExternalLinks(True)
             dialog.exec_()
             sys.exit(1)
         if BaseFrontend.debconffilter_done(self, dbfilter):
@@ -1175,6 +1148,7 @@ class Wizard(BaseFrontend):
         if finished_step == last_page and not self.backup:
             self.finished_pages = True
             if self.finished_installing or self.oem_user_config:
+                self.debconf_progress_info('')
                 self.ui.progressBar.show()
                 dbfilter = plugininstall.Install(self)
                 dbfilter.start(auto_process=True)
@@ -1209,6 +1183,8 @@ class Wizard(BaseFrontend):
             if self.finished_pages:
                 dbfilter = plugininstall.Install(self)
                 dbfilter.start(auto_process=True)
+            else:
+                self.ui.progressBar.hide()
 
         elif finished_step == 'ubiquity.components.plugininstall':
             self.installing = False
