@@ -7,7 +7,7 @@
 # Authors:
 #
 # - Colin Watson <cjwatson@ubuntu.com>
-# - Evan Dandrea <evand@ubuntu.com>
+# - Evan Dandrea <ev@ubuntu.com>
 # - Roman Shtylman <shtylman@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -24,62 +24,57 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-import os, re
+import os
+import re
+
+import debconf
 
 from ubiquity import validation
-from ubiquity.misc import execute, execute_root, dmimodel
+from ubiquity import misc
 from ubiquity import plugin
-import debconf
 
 NAME = 'usersetup'
 AFTER = 'console_setup'
 WEIGHT = 10
 
 def check_hostname(hostname):
-    """Returns a newline separated string of reasons why the hostname is
-    invalid."""
-    # TODO: i18n
-    e = []
-    # Ahem.  We can cheat here by inserting newlines where needed.  Hopefully
-    # by the time we translate this, GTK+ will have decent layout management.
-    for result in validation.check_hostname(unicode(hostname)):
+    """Returns a list of reasons why the hostname is invalid."""
+    errors = []
+    for result in validation.check_hostname(misc.utf8(hostname)):
         if result == validation.HOSTNAME_LENGTH:
-            e.append("Must be between 1 and 63 characters long.")
+            errors.append('hostname_error_length')
         elif result == validation.HOSTNAME_BADCHAR:
-            e.append("May only contain letters, digits,\nhyphens, and dots.")
+            errors.append('hostname_error_badchar')
         elif result == validation.HOSTNAME_BADHYPHEN:
-            e.append("May not start or end with a hyphen.")
+            errors.append('hostname_error_badhyphen')
         elif result == validation.HOSTNAME_BADDOTS:
-            e.append('May not start or end with a dot,\n'
-                     'or contain the sequence "..".')
-    return "\n".join(e)
+            errors.append('hostname_error_baddots')
+    return errors
 
 def check_username(username):
-    """Returns a newline separated string of reasons why the username is
-    invalid."""
-    # TODO: i18n
-    e = []
-    # Ahem.  We can cheat here by inserting newlines where needed.  Hopefully
-    # by the time we translate this, GTK+ will have decent layout management.
+    """Returns a list of reasons why the username is invalid."""
     if username:
         if not re.match('[a-z]', username[0]):
-            e.append("Must start with a lower-case letter.")
+            return ['username_error_badfirstchar']
         # Technically both these conditions might hold.  However, the common
         # case seems to be that somebody starts typing their name beginning
         # with an upper-case letter, and it's probably sufficient to just
         # issue the first error in that case.
         elif not re.match('^[-a-z0-9_]+$', username):
-            e.append("May only contain lower-case letters,\n"
-                     "digits, hyphens, and underscores.")
-    return "\n".join(e)
+            return ['username_error_badchar']
+    return []
+
+def make_error_string(controller, errors):
+    """Returns a newline-separated string of translated error reasons."""
+    return "\n".join([controller.get_string(error) for error in errors])
 
 class PageBase(plugin.PluginUI):
     def __init__(self):
-        self.suffix = dmimodel()
+        self.suffix = misc.dmimodel()
         if self.suffix:
             self.suffix = '-%s' % self.suffix
         else:
-            if execute("laptop-detect"):
+            if misc.execute("laptop-detect"):
                 self.suffix = '-laptop'
             else:
                 self.suffix = '-desktop'
@@ -161,7 +156,10 @@ class PageBase(plugin.PluginUI):
 class PageGtk(PageBase):
     plugin_title = 'ubiquity/text/userinfo_heading_label'
     def __init__(self, controller, *args, **kwargs):
+        from gi.repository import Gio, Gtk
+
         PageBase.__init__(self, *args, **kwargs)
+        self.resolver = Gio.Resolver.get_default()
         self.controller = controller
         self.username_changed_id = None
         self.hostname_changed_id = None
@@ -169,10 +167,9 @@ class PageGtk(PageBase):
         self.hostname_edited = False
         self.hostname_timeout_id = 0
 
-        import gtk
-        builder = gtk.Builder()
+        builder = Gtk.Builder()
         self.controller.add_builder(builder)
-        builder.add_from_file('/usr/share/ubiquity/gtk/stepUserInfo.ui')
+        builder.add_from_file(os.path.join(os.environ['UBIQUITY_GLADE'], 'stepUserInfo.ui'))
         builder.connect_signals(self)
         self.page = builder.get_object('stepUserInfo')
         self.username = builder.get_object('username')
@@ -199,7 +196,7 @@ class PageGtk(PageBase):
         # okay check icon and the hostname error messages.
         paddingbox = builder.get_object('paddingbox')
         def func(box):
-            box.parent.child_set_property(box, 'expand', False)
+            box.get_parent().child_set_property(box, 'expand', False)
             box.set_size_request(box.get_allocation().width / 2, -1)
         paddingbox.connect('realize', func)
 
@@ -222,9 +219,10 @@ class PageGtk(PageBase):
             self.hostname_edited = True
             self.login_vbox.hide()
             # The UserSetup component takes care of preseeding passwd/user-uid.
-            execute_root('apt-install', 'oem-config-gtk',
-                                        'oem-config-slideshow-ubuntu')
+            misc.execute_root('apt-install', 'oem-config-gtk',
+                                             'oem-config-slideshow-ubuntu')
 
+        self.resolver_ok = True
         self.plugin_widgets = self.page
 
     def plugin_translate(self, lang):
@@ -236,10 +234,10 @@ class PageGtk(PageBase):
         pasw = self.controller.get_string('password_inactive_label', lang)
         vpas = self.controller.get_string('password_again_inactive_label',
                                           lang)
-        self.username.set_label(user)
-        self.fullname.set_label(full)
-        self.password.set_label(pasw)
-        self.verified_password.set_label(vpas)
+        self.username.set_placeholder_text(user)
+        self.fullname.set_placeholder_text(full)
+        self.password.set_placeholder_text(pasw)
+        self.verified_password.set_placeholder_text(vpas)
 
     # Functions called by the Page.
 
@@ -318,9 +316,11 @@ class PageGtk(PageBase):
         if (widget is not None and widget.get_name() == 'fullname' and
             not self.username_edited):
             self.username.handler_block(self.username_changed_id)
-            new_username = widget.get_text().split(' ')[0]
+            new_username = misc.utf8(widget.get_text().split(' ')[0])
             new_username = new_username.encode('ascii', 'ascii_transliterate')
             new_username = new_username.lower()
+            new_username = re.sub('^[^a-z]+', '', new_username)
+            new_username = re.sub('[^-a-z0-9_]', '', new_username)
             self.username.set_text(new_username)
             self.username.handler_unblock(self.username_changed_id)
         elif (widget is not None and widget.get_name() == 'username' and
@@ -343,9 +343,9 @@ class PageGtk(PageBase):
 
         text = self.username.get_text()
         if text:
-            error_msg = check_username(text)
-            if error_msg:
-                self.username_error(error_msg)
+            errors = check_username(text)
+            if errors:
+                self.username_error(make_error_string(self.controller, errors))
                 complete = False
             else:
                 self.username_ok.show()
@@ -389,9 +389,9 @@ class PageGtk(PageBase):
         txt = self.hostname.get_text()
         self.hostname_ok.show()
         if txt:
-            error_msg = check_hostname(txt)
-            if error_msg:
-                self.hostname_error(error_msg)
+            errors = check_hostname(txt)
+            if errors:
+                self.hostname_error(make_error_string(self.controller, errors))
                 complete = False
                 self.hostname_ok.hide()
             else:
@@ -403,25 +403,26 @@ class PageGtk(PageBase):
             self.hostname_error_label.hide()
 
         self.controller.allow_go_forward(complete)
-    
+
     def on_username_changed(self, widget):
         self.username_edited = (widget.get_text() != '')
 
     def on_hostname_changed(self, widget):
         self.hostname_edited = (widget.get_text() != '')
 
-        # Let's not call this every time the user presses a key.
-        import gobject
-        if self.hostname_timeout_id:
-            gobject.source_remove(self.hostname_timeout_id)
-        self.hostname_timeout_id = gobject.timeout_add(300,
-                                        self.hostname_timeout, widget)
+        if not 'UBIQUITY_AUTOMATIC' in os.environ:
+            # Let's not call this every time the user presses a key.
+            from gi.repository import GObject
+            if self.hostname_timeout_id:
+                GObject.source_remove(self.hostname_timeout_id)
+            self.hostname_timeout_id = GObject.timeout_add(300,
+                                            self.hostname_timeout, widget)
 
-    def lookup_result(self, resolver, result):
-        import glib
+    def lookup_result(self, resolver, result, unused):
+        from gi.repository import GObject
         try:
             resolver.lookup_by_name_finish(result)
-        except glib.GError:
+        except GObject.GError:
             pass
         else:
             # FIXME: i18n
@@ -429,12 +430,27 @@ class PageGtk(PageBase):
             self.hostname_ok.hide()
 
     def hostname_timeout(self, widget):
-        import gio
-        if self.hostname_ok.get_property('visible'):
-            res = gio.resolver_get_default()
+        if self.hostname_ok.get_property('visible') and self.resolver_ok :
             hostname = widget.get_text()
             for host in (hostname, '%s.local' % hostname):
-                res.lookup_by_name_async(self.lookup_result, host)
+                self.resolver.lookup_by_name_async(
+                    host, None, self.lookup_result, None)
+
+    def detect_bogus_result(self, hostname = 'xyzzy_does_not_exist'):
+        # bug 760884
+        # On networks where DNS fakes a response for unknown hosts,
+        # don't display a warning for hostnames that already exist.
+        self.resolver.lookup_by_name_async(
+            hostname, None, self.bogus_lookup_result, None)
+
+    def bogus_lookup_result(self, resolver, result, unused):
+        from gi.repository import GObject
+        try:
+            resolver.lookup_by_name_finish(result)
+        except GObject.GError:
+            self.resolver_ok = True
+        else:
+            self.resolver_ok = False
 
     def on_authentication_toggled(self, w):
         if w == self.login_auto and w.get_active():
@@ -442,7 +458,7 @@ class PageGtk(PageBase):
         elif w == self.login_encrypt and w.get_active():
             # TODO why is this so slow to activate the login_pass radio button
             # when checking encrypted home?
-            self.login_pass.activate()
+            self.login_pass.set_active(True)
 
 class PageKde(PageBase):
     plugin_breadcrumb = 'ubiquity/text/breadcrumb_user'
@@ -452,7 +468,7 @@ class PageKde(PageBase):
         self.controller = controller
 
         from PyQt4 import uic
-        from PyKDE4.kdeui import KIconLoader
+        from PyQt4.QtGui import QPixmap
 
         self.plugin_widgets = uic.loadUi('/usr/share/ubiquity/qt/stepUserSetup.ui')
         self.page = self.plugin_widgets
@@ -476,10 +492,10 @@ class PageKde(PageBase):
             self.page.hostname.setText('oem%s' % self.suffix)
 
             # The UserSetup component takes care of preseeding passwd/user-uid.
-            execute_root('apt-install', 'oem-config-kde')
+            misc.execute_root('apt-install', 'oem-config-kde')
 
-        iconLoader = KIconLoader()
-        warningIcon = iconLoader.loadIcon("dialog-warning", KIconLoader.Desktop)
+        warningIcon = QPixmap(
+                     "/usr/share/icons/oxygen/48x48/status/dialog-warning.png")
         self.page.fullname_error_image.setPixmap(warningIcon)
         self.page.username_error_image.setPixmap(warningIcon)
         self.page.password_error_image.setPixmap(warningIcon)
@@ -549,7 +565,7 @@ class PageKde(PageBase):
 
     def set_encrypt_home(self, value):
         self.page.login_encrypt.setChecked(value)
-    
+
     def set_force_encrypt_home(self, value):
         self.page.login_encrypt.setDisabled(value)
         self.page.login_auto.setDisabled(value)
@@ -724,11 +740,15 @@ class Page(plugin.Plugin):
         except debconf.DebconfError:
             empty = False
         self.ui.set_allow_password_empty(empty)
-        
+
         # We need to call info_loop as we switch to the page so the next button
         # gets disabled.
         self.ui.info_loop(None)
 
+        # Trigger the bogus DNS server detection
+        if not 'UBIQUITY_AUTOMATIC' in os.environ and hasattr(self.ui, 'detect_bogus_result'):
+            self.ui.detect_bogus_result()
+
         # We intentionally don't listen to passwd/auto-login or
         # user-setup/encrypt-home because we don't want those alone to force
         # the page to be shown, if they're the only questions not preseeded.
@@ -740,7 +760,10 @@ class Page(plugin.Plugin):
             return (['/usr/lib/ubiquity/user-setup/user-setup-ask-oem'],
                     questions, environ)
         else:
-            return (['/usr/lib/ubiquity/user-setup/user-setup-ask', '/target'],
+            # TODO: It would be neater to use a wrapper script.
+            return (['sh', '-c',
+                     '/usr/lib/ubiquity/user-setup/user-setup-ask /target && '
+                     '/usr/share/ubiquity/user-setup-encrypted-swap'],
                     questions)
 
     def set(self, question, value):
@@ -776,11 +799,11 @@ class Page(plugin.Plugin):
         hostname = self.ui.get_hostname()
 
         # check if the hostname had errors
-        error_msg = check_hostname(hostname)
+        errors = check_hostname(hostname)
 
         # showing warning message is error is set
-        if len(error_msg) != 0:
-            self.ui.hostname_error(error_msg)
+        if errors:
+            self.ui.hostname_error(make_error_string(self.ui.controller, errors))
             self.done = False
             self.enter_ui_loop()
             return
@@ -811,8 +834,11 @@ class Install(plugin.InstallPlugin):
             environ = {'OVERRIDE_SYSTEM_USER': '1'}
             return (['/usr/lib/ubiquity/user-setup/user-setup-apply'], [], environ)
         else:
+            environ = {}
+            if os.path.exists('/var/lib/ubiquity/encrypted-swap'):
+                environ['OVERRIDE_ALREADY_ENCRYPTED_SWAP'] = '1'
             return (['/usr/lib/ubiquity/user-setup/user-setup-apply', '/target'],
-                    [])
+                    [], environ)
 
     def error(self, priority, question):
         self.ui.error_dialog(self.description(question),
